abstract class List<E> {
  abstract <R> R accept(ListVisitor<E, R> v);
  ConsList<E> cons(E elt) { return new ConsList<E>(elt, this); } 
  abstract String toStringHelp();
}

interface ListVisitor<E, R> {
  R forEmptyList(EmptyList<E> host);
  R forConsList(ConsList<E> host);
}

class EmptyList<E> extends List<E> { 
  <R> R accept(ListVisitor<E, R> v) { return v.forEmptyList(this); }
  String toStringHelp() { return ""; }
  public String toString() { return "()"; }
  public boolean equals(Object other) { return getClass() == other.getClass(); }
}

class ConsList<E> extends List<E> {
  E first;
  List<E> rest;
  ConsList(E f, List<E> r) {
    first = f;
    rest = r;
  }
  <R> R accept(ListVisitor<E, R> v) { return v.forConsList(this); }
  String toStringHelp() { return " " + first + rest.toStringHelp(); }
  public String toString() { return "(" + first + rest.toStringHelp() + ")"; }
  public boolean equals(Object o) { 
    return getClass() == o.getClass() && first.equals(((ConsList) o).first) && rest.equals(((ConsList) o).rest);
  }
}

class InsertVisitor<E extends Number> implements ListVisitor<E, List<E>> {
  E arg;
  InsertVisitor(E n) { arg = n; }
  public List<E> forEmptyList(EmptyList<E> host) { return new EmptyList<E>().cons(arg); }
  public List<E> forConsList(ConsList<E> host) {
    if (arg.doubleValue() <= host.first.doubleValue()) return host.cons(arg);
    return host.rest.accept(this).cons(host.first);
  }
}

class MergeVisitor<E extends Number> implements ListVisitor<E, List<E>> {
  List<E> other;
  MergeVisitor(List<E> o) { other = o; }
  public List<E> forEmptyList(EmptyList<E> host) { return other; }
  public List<E> forConsList(ConsList<E> host) { return other.accept(new MergeHelpVisitor<E>(host)); }
}

class MergeHelpVisitor<E extends Number> implements ListVisitor<E, List<E>> {
  ConsList<E> other;
  MergeHelpVisitor(ConsList<E> o) { other = o; }
  public List<E> forEmptyList(EmptyList<E> host) { return other; }
  public List<E> forConsList(ConsList<E> host) { 
    E o1 = other.first;
    E h1 = host.first;
    List<E> oRest = other.rest;
    List<E> hRest = host.rest;
    if (o1.doubleValue() <= h1.doubleValue()) return oRest.accept(new MergeHelpVisitor<E>(host)).cons(o1);
    else return hRest.accept(this).cons(h1);
  }
}
  