// Language Level Converter line number map: dj*->java. Entries: 90
//     1->14        2->15        3->16        4->17        5->18        6->19        7->20        8->21   
//     9->22       10->23       11->24       12->25       13->26       14->27       15->28       16->29   
//    17->30       18->31       19->32       20->33       21->64       22->65       23->66       24->67   
//    25->68       26->69       27->70       28->71       29->72       30->73       31->74       32->75   
//    33->76       34->102      35->103      36->104      37->105      38->106      39->107      40->108  
//    41->109      42->110      43->111      44->112      45->113      46->114      47->115      48->116  
//    49->117      50->118      51->119      52->120      53->121      54->169      55->170      56->171  
//    57->172      58->173      59->174      60->175      61->176      62->177      63->178      64->217  
//    65->218      66->219      67->220      68->221      69->222      70->223      71->262      72->263  
//    73->264      74->265      75->266      76->267      77->268      78->269      79->270      80->271  
//    81->310      82->311      83->312      84->313      85->314      86->315      87->316      88->317  
//    89->318      90->319  
/** IntList ::= EmptyIntList | ConsIntList(int, IntList).*/
interface IntListVisitor {
  public Object forEmptyIntList(final EmptyIntList host);
  public Object forConsIntList(final ConsIntList host);
}

abstract class IntList {
  /** Hook for applying visitor operations. */
  public abstract Object accept(final IntListVisitor v);
  /** Sorts this IntList into ascending (non-descending) order using list insertion. */
  public abstract IntList sort();
  /** Adds the int n to the front of this IntList. */
  public IntList cons(final int n) { return new ConsIntList(n, this); }
  /** Inserts n in proper order in this IntList, assuming this is sorted in ascending order. */
  public abstract IntList insert(final int n);
  /** Merges other with this IntList assuming both are sored in ascending (non-descending) order. */
  public abstract IntList merge(final IntList other);
  /** Merges other with this IntList assuming both are sored in ascending (non-descending) order, other is non-empty. */
  public abstract IntList mergeHelp(final ConsIntList other);

  /** This method is automatically generated by the Language Level Converter. */
  public IntList() {
    super();
  }

  /** This method is automatically generated by the Language Level Converter. */
  public java.lang.String toString() {
    return getClass().getName() + "(" + 
        ")";
  }

  /** This method is automatically generated by the Language Level Converter. */
  public boolean equals(java.lang.Object o) {
    if (this == o) {
      return true;
    }
    else if ((o == null) || (! o.getClass().equals(getClass()))) {
      return false;
    }
    else {
      IntList cast = ((IntList) o);
      return true;
    }
  }

  /** This method is automatically generated by the Language Level Converter. */
  public int hashCode() {
    return getClass().hashCode();
  }
}

class EmptyIntList extends IntList { 
  /** Field containing the only (singleton) EmptyIntList. */
  public final static EmptyIntList ONLY = new EmptyIntList();
  /** Constructor used to create ONLY (this). */
  private EmptyIntList() { }

  public Object accept(final IntListVisitor v) { return v.forEmptyIntList(this); }
  public IntList sort() { return this; }
  public IntList insert(final int n) { return cons(n); }
  public IntList merge(final IntList other) { return other; }
  public IntList mergeHelp(final ConsIntList other) { return other; }

  /** This method is automatically generated by the Language Level Converter. */
  public java.lang.String toString() {
    return getClass().getName() + "(" + 
        ")";
  }

  /** This method is automatically generated by the Language Level Converter. */
  public boolean equals(java.lang.Object o) {
    if (this == o) {
      return true;
    }
    else if ((o == null) || (! o.getClass().equals(getClass()))) {
      return false;
    }
    else {
      EmptyIntList cast = ((EmptyIntList) o);
      return true;
    }
  }

  /** This method is automatically generated by the Language Level Converter. */
  public int hashCode() {
    return getClass().hashCode();
  }
}

class ConsIntList extends IntList {
  /** The first element of this. */
  private final int first;
  /** The remaining elements of this. */
  private final IntList rest;
  public Object accept(final IntListVisitor v) { return v.forConsIntList(this); }
  public IntList sort() { return rest.sort().insert(first); }
  public IntList insert(final int n) {
    if (n <= first) return cons(n);
    else return rest.insert(n).cons(first);
  }
  public IntList merge(final IntList other) {
    if (other.equals(EmptyIntList.ONLY)) return this;
    else return other.mergeHelp(this); }
  public IntList mergeHelp(final ConsIntList other) {
    if (first <= other.first()) return rest.mergeHelp(other).cons(first);
    else return other.rest().merge(this).cons(other.first());
  }

  /** This method is automatically generated by the Language Level Converter. */
  public ConsIntList(int first, IntList rest) {
    super();
    this.first = first;
    this.rest = rest;
  }

  /** This method is automatically generated by the Language Level Converter. */
  public int first() {
    return first;
  }

  /** This method is automatically generated by the Language Level Converter. */
  public IntList rest() {
    return rest;
  }

  /** This method is automatically generated by the Language Level Converter. */
  public java.lang.String toString() {
    return getClass().getName() + "(" + 
        first() + ", " + 
        rest() + 
        ")";
  }

  /** This method is automatically generated by the Language Level Converter. */
  public boolean equals(java.lang.Object o) {
    if (this == o) {
      return true;
    }
    else if ((o == null) || (! o.getClass().equals(getClass()))) {
      return false;
    }
    else {
      ConsIntList cast = ((ConsIntList) o);
      return (first() == cast.first()) && 
          (rest() != null && rest().equals(cast.rest()));
    }
  }

  /** This method is automatically generated by the Language Level Converter. */
  public int hashCode() {
    return getClass().hashCode() ^ 
        first() ^ 
        (rest() == null ? 0 : rest().hashCode());
  }
}

class InsertVisitor implements IntListVisitor {
  private final int elt;

  public IntList forEmptyIntList(final EmptyIntList host) { return host.cons(elt); }
  public IntList forConsIntList(final ConsIntList host) {
    if (elt <= host.first()) return host.cons(elt);
    else return ((IntList)(host.rest().accept(this))).cons(host.first());
  }

  /** This method is automatically generated by the Language Level Converter. */
  public InsertVisitor(int elt) {
    super();
    this.elt = elt;
  }

  /** This method is automatically generated by the Language Level Converter. */
  public int elt() {
    return elt;
  }

  /** This method is automatically generated by the Language Level Converter. */
  public java.lang.String toString() {
    return getClass().getName() + "(" + 
        elt() + 
        ")";
  }

  /** This method is automatically generated by the Language Level Converter. */
  public boolean equals(java.lang.Object o) {
    if (this == o) {
      return true;
    }
    else if ((o == null) || (! o.getClass().equals(getClass()))) {
      return false;
    }
    else {
      InsertVisitor cast = ((InsertVisitor) o);
      return (elt() == cast.elt());
    }
  }

  /** This method is automatically generated by the Language Level Converter. */
  public int hashCode() {
    return getClass().hashCode() ^ 
        elt();
  }
}

class MergeVisitor implements IntListVisitor{
  private final IntList other;
  
  public IntList forEmptyIntList(final EmptyIntList host) { return other; }
  public IntList forConsIntList(final ConsIntList host) { return (IntList) other.accept(new MergeHelpVisitor(host)); }

  /** This method is automatically generated by the Language Level Converter. */
  public MergeVisitor(IntList other) {
    super();
    this.other = other;
  }

  /** This method is automatically generated by the Language Level Converter. */
  public IntList other() {
    return other;
  }

  /** This method is automatically generated by the Language Level Converter. */
  public java.lang.String toString() {
    return getClass().getName() + "(" + 
        other() + 
        ")";
  }

  /** This method is automatically generated by the Language Level Converter. */
  public boolean equals(java.lang.Object o) {
    if (this == o) {
      return true;
    }
    else if ((o == null) || (! o.getClass().equals(getClass()))) {
      return false;
    }
    else {
      MergeVisitor cast = ((MergeVisitor) o);
      return (other() != null && other().equals(cast.other()));
    }
  }

  /** This method is automatically generated by the Language Level Converter. */
  public int hashCode() {
    return getClass().hashCode() ^ 
        (other() == null ? 0 : other().hashCode());
  }
}
  
class MergeHelpVisitor implements IntListVisitor{
  private final ConsIntList other;
  
  public IntList forEmptyIntList(final EmptyIntList host) { return other; }
  public IntList forConsIntList(final ConsIntList host) { 
    if (host.first() <= other.first()) return ((IntList)(host.rest().accept(this))).cons(host.first());
    return ((IntList)(other.rest().accept(new MergeHelpVisitor(host)))).cons(other.first());
  }

  /** This method is automatically generated by the Language Level Converter. */
  public MergeHelpVisitor(ConsIntList other) {
    super();
    this.other = other;
  }

  /** This method is automatically generated by the Language Level Converter. */
  public ConsIntList other() {
    return other;
  }

  /** This method is automatically generated by the Language Level Converter. */
  public java.lang.String toString() {
    return getClass().getName() + "(" + 
        other() + 
        ")";
  }

  /** This method is automatically generated by the Language Level Converter. */
  public boolean equals(java.lang.Object o) {
    if (this == o) {
      return true;
    }
    else if ((o == null) || (! o.getClass().equals(getClass()))) {
      return false;
    }
    else {
      MergeHelpVisitor cast = ((MergeHelpVisitor) o);
      return (other() != null && other().equals(cast.other()));
    }
  }

  /** This method is automatically generated by the Language Level Converter. */
  public int hashCode() {
    return getClass().hashCode() ^ 
        (other() == null ? 0 : other().hashCode());
  }
}

class SortVisitor implements IntListVisitor {
  public final static SortVisitor ONLY = new SortVisitor();
  private SortVisitor() { }
  public IntList forEmptyIntList(final EmptyIntList host) { return host; }
  public IntList forConsIntList(final ConsIntList host) {
    final IntList sortedRest = (IntList) (host.rest().accept(this));
    return (IntList) sortedRest.accept(new InsertVisitor(host.first()));
  }

  /** This method is automatically generated by the Language Level Converter. */
  public java.lang.String toString() {
    return getClass().getName() + "(" + 
        ")";
  }

  /** This method is automatically generated by the Language Level Converter. */
  public boolean equals(java.lang.Object o) {
    if (this == o) {
      return true;
    }
    else if ((o == null) || (! o.getClass().equals(getClass()))) {
      return false;
    }
    else {
      SortVisitor cast = ((SortVisitor) o);
      return true;
    }
  }

  /** This method is automatically generated by the Language Level Converter. */
  public int hashCode() {
    return getClass().hashCode();
  }
}
