import java.util.*;

interface IntListVisitor { 
  public IntList forEmptyIntList(final EmptyIntList host);  
  public IntList forConsIntList(final ConsIntList host);
}

abstract class IntList {
  /** Applies the visitor v to this. */
  public abstract IntList accept(final IntListVisitor v);
  
  /** Adds the int n to the front of this. */
  public IntList cons(final int n) { return new ConsIntList(n, this); }

  /** This method is automatically generated by the Language Level Converter. */
  public IntList() {
    super();
  }

  /** This method is automatically generated by the Language Level Converter. */
  public java.lang.String toString() {
    return getClass().getName() + "(" + 
        ")";
  }

  /** This method is automatically generated by the Language Level Converter. */
  public boolean equals(java.lang.Object o) {
    if (this == o) {
      return true;
    }
    else if ((o == null) || (! o.getClass().equals(getClass()))) {
      return false;
    }
    else {
      IntList cast = ((IntList) o);
      return true;
    }
  }

  /** This method is automatically generated by the Language Level Converter. */
  public int hashCode() {
    return getClass().hashCode();
  }
}

class EmptyIntList extends IntList { 
  public final static EmptyIntList ONLY = new EmptyIntList();
  private EmptyIntList() { }
  public IntList accept(final IntListVisitor v) { return v.forEmptyIntList(this); }

  /** This method is automatically generated by the Language Level Converter. */
  public java.lang.String toString() {
    return getClass().getName() + "(" + 
        ")";
  }

  /** This method is automatically generated by the Language Level Converter. */
  public boolean equals(java.lang.Object o) {
    if (this == o) {
      return true;
    }
    else if ((o == null) || (! o.getClass().equals(getClass()))) {
      return false;
    }
    else {
      EmptyIntList cast = ((EmptyIntList) o);
      return true;
    }
  }

  /** This method is automatically generated by the Language Level Converter. */
  public int hashCode() {
    return getClass().hashCode();
  }
}

class ConsIntList extends IntList {
  /** first element of this. */
  private final int first;
  /** remaining elements of this. */
  private final IntList rest;
  public IntList accept(final IntListVisitor v) { return v.forConsIntList(this); }

  /** This method is automatically generated by the Language Level Converter. */
  public ConsIntList(int first, IntList rest) {
    super();
    this.first = first;
    this.rest = rest;
  }

  /** This method is automatically generated by the Language Level Converter. */
  public int first() {
    return first;
  }

  /** This method is automatically generated by the Language Level Converter. */
  public IntList rest() {
    return rest;
  }

  /** This method is automatically generated by the Language Level Converter. */
  public java.lang.String toString() {
    return getClass().getName() + "(" + 
        first() + ", " + 
        rest() + 
        ")";
  }

  /** This method is automatically generated by the Language Level Converter. */
  public boolean equals(java.lang.Object o) {
    if (this == o) {
      return true;
    }
    else if ((o == null) || (! o.getClass().equals(getClass()))) {
      return false;
    }
    else {
      ConsIntList cast = ((ConsIntList) o);
      return (first() == cast.first()) && 
          (rest() != null && rest().equals(cast.rest()));
    }
  }

  /** This method is automatically generated by the Language Level Converter. */
  public int hashCode() {
    return getClass().hashCode() ^ 
        first() ^ 
        (rest() == null ? 0 : rest().hashCode());
  }
}

class UpSortVisitor implements IntListVisitor {
  public IntList forEmptyIntList(final EmptyIntList host) { return host; }
  public IntList forConsIntList(final ConsIntList host) {
    return host.rest().accept(this).accept(new InsertVisitor(host.first())); 
  }

  /** This method is automatically generated by the Language Level Converter. */
  public UpSortVisitor() {
    super();
  }

  /** This method is automatically generated by the Language Level Converter. */
  public java.lang.String toString() {
    return getClass().getName() + "(" + 
        ")";
  }

  /** This method is automatically generated by the Language Level Converter. */
  public boolean equals(java.lang.Object o) {
    if (this == o) {
      return true;
    }
    else if ((o == null) || (! o.getClass().equals(getClass()))) {
      return false;
    }
    else {
      UpSortVisitor cast = ((UpSortVisitor) o);
      return true;
    }
  }

  /** This method is automatically generated by the Language Level Converter. */
  public int hashCode() {
    return getClass().hashCode();
  }
}

class InsertVisitor implements IntListVisitor {
  private final int elt;
  public IntList forEmptyIntList(final EmptyIntList host) { return host.cons(elt); } 
  public IntList forConsIntList(final ConsIntList host) {
    final int first = host.first();
    final IntList rest = host.rest();
    if (elt <= first) return host.cons(elt);
    return rest.accept(this).cons(first);
  }

  /** This method is automatically generated by the Language Level Converter. */
  public InsertVisitor(int elt) {
    super();
    this.elt = elt;
  }

  /** This method is automatically generated by the Language Level Converter. */
  public int elt() {
    return elt;
  }

  /** This method is automatically generated by the Language Level Converter. */
  public java.lang.String toString() {
    return getClass().getName() + "(" + 
        elt() + 
        ")";
  }

  /** This method is automatically generated by the Language Level Converter. */
  public boolean equals(java.lang.Object o) {
    if (this == o) {
      return true;
    }
    else if ((o == null) || (! o.getClass().equals(getClass()))) {
      return false;
    }
    else {
      InsertVisitor cast = ((InsertVisitor) o);
      return (elt() == cast.elt());
    }
  }

  /** This method is automatically generated by the Language Level Converter. */
  public int hashCode() {
    return getClass().hashCode() ^ 
        elt();
  }
}
